/*--------------------------------------------------------------------------
 * Copyright (C) 2003 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  usbport.C - This utility uses checks the port mapping for DS2490 based
 *              USB 1-Wire adapters 
 *
 *  Compiler:  Microsoft Visual C 
 *  Version:   1.01
 *
 */

#define TMEXUTIL

#include "iBTMEXcw.H"   

#define LEN 1000

/* Local Function Prototypes */
void Exit_Prog(char *, short);
short ReadPortID(short Prt, short Tp, char *ROM, long *last);

/* Global Variables to hold list information */
short PortType, PortNum;
uchar state_buffer[5120];
uchar buf[8000];
ushort CRC16;
short ROM[9];

/*----------------------------------------------------------------------
 *  This is the Main routine 
 */
void main(short argc, char **argv)
{
   short writelen=0,i,usbdefault=0,prt,gotaddress;
   char tstr[200],id[255],tmp[256];       
   long session_handle,last;
   Specification spec;

   // check for clear the registry flag
   if (argc >= 2)
   {
      if ((argv[1][0] == 'c') || (argv[1][0] == 'C'))
      {
         printf("\nClearing the USB adapter ID associations in the registry...\n"); 
         sprintf(tmp,"Software\\Dallas Semiconductor\\iButton TMEX\\PortID\\Type6");
         if (RegDeleteKey(HKEY_LOCAL_MACHINE,tmp) == ERROR_SUCCESS)
            Exit_Prog("Registry update complete.\n", NORMAL_EXIT);
         else
            Exit_Prog("ERROR updating registry.\n", ERROR_REG);
      }
   }
 
   // read the default PortNum and PortType 
   if (!TMReadDefaultPort(&PortNum,&PortType))
       Exit_Prog("ERROR, Could not read the default PortNum and PortType from registry!\n",
               NO_DRIVERS);

   if (PortType == 6)
      usbdefault = TRUE;

   // hard write it to type 6 USB, on default port
   PortType = 6;

   // print a header   
   printf("\nDS2490 (DS9490) USB Port Lookup Version 1.01\n"
         "Dallas Semiconductor Corporation / Maxim\n"); 

    printf("\nusage: usbport <clear>\n"
          "   <clear> optional clear all USB adapter ID associations\n"
          "   version 1.01\n\n");
   
   printf("Port number: %d    port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("TMEX main driver:\n %s\n",tstr);
   printf("TMEX adapter driver, Type%d:\n",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
      Exit_Prog("No Hardware Driver for this type found!",NO_DRIVERS);
   printf(" %s\n\n",tstr);

   printf("Port  Present  Adapter ID    Default \n");
   printf("=====================================\n");
   
   // loop through all ports
   for (prt = 0; prt < 16; prt++)
   {
      gotaddress = FALSE;

      printf(" %02d      ",prt);

      // attempt to get a session 
      session_handle = TMExtendedStartSession(prt,PortType,NULL);
      if (session_handle > 0)  
      {
         if (TMSetup(session_handle) == 1)
         {
            printf("X   ");

            if (TMGetAdapterSpec(session_handle,&spec) == 1)
            {
               if (strlen(spec.dscrptn) > 43)
               {
                  for (i = 0; i < 16; i++)
                     printf("%c",spec.dscrptn[26+i]);
               }
               else
                  printf("     --         ");
               gotaddress = TRUE;
            }
            else
               printf("       ");
         }
         else 
            printf("    ");

         TMEndSession(session_handle);
      }

      if (!gotaddress)
      {
         // ID
         if (ReadPortID(prt, PortType, id, &last))
            printf("%s",id);
         else
            printf("                ");
      }

      // default
      if (usbdefault && (prt == PortNum))
         printf("   X   ");
      else   
         printf("       ");


      printf("\n");
   }

   printf("\nHit any key to exit\n");
   getch();
}


/*--------------------------------------------------------------------------
 *  Prints a message, beeps and exits the program.
 */
void Exit_Prog(char *msg, short errcd)
{
   /* print the message left justified */
   printf("\r%s",msg);
   
   /*  error beep if return code is not 0 */
   if (errcd != NORMAL_EXIT)
     MessageBeep(MB_ICONEXCLAMATION);
   
   /*  return error code passed */
   exit(errcd);
}


//---------------------------------------------------------------------------
//  Description:
//     Read the port ID number from the registry, return in a ROM array.
//
short ReadPortID(short Prt, short Tp, char *ROM, long *last)
{
   HKEY GlobKey;
   DWORD dwType,size;
   char ReturnString[256], tmp[256];
   short rt = FALSE;
   long tmpLast;

   // read the default PortNum and PortType from the registry 
   sprintf(tmp,"Software\\Dallas Semiconductor\\iButton TMEX\\PortID\\Type%d",Tp);
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,tmp,0,
         KEY_READ,&GlobKey) == ERROR_SUCCESS)
   {
      // attempt to read the PortNum 
      size = 255;
      sprintf(tmp,"Port%d",Prt);
      sprintf(ReturnString,"");
      if (RegQueryValueEx(GlobKey,tmp,NULL,&dwType,
          &ROM[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) && (size >= 17))
            rt = TRUE;
      }
      
      // read the 'last' pointer to last port updated
      size = sizeof(tmpLast);
      if (RegQueryValueEx(GlobKey,"Last",NULL,&dwType,
          (uchar *)(&tmpLast),&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_DWORD) && (size > 0))
            *last = tmpLast;
         else
            *last = 1;
      }

      // close the key
      RegCloseKey(GlobKey);
   }

   return rt;
}
